import importlib.metadata as metadata
import os
import platform
import sys

from typing import Any, BinaryIO, Callable, List, Mapping, Optional, cast, Union

import struct
from types import SimpleNamespace
from dataclasses import dataclass
import functools
import weakref

from wasmtime import Engine, Instance, Memory, Store, Module, Linker, WasiConfig
import wasmtime

from one_sdk.handle_map import HandleMap
from one_sdk.sf_host import Ptr, Size, link as sf_host_link
from one_sdk.error import HostError, ErrorCode, PerformError, ValidationError, UnexpectedError, UninitializedError, WasiError, WasiErrno
from one_sdk.platform import PythonFilesystem, PythonNetwork, PythonPersistence, DeferredHttpResponse, HttpResponse

# TODO: TypeAlias - needs 3.10
SecurityValuesMap = Mapping[str, Mapping[str, str]]

class WasiMemory:
	"""Pointer to Wasi Memory - do not store this between calls to WASM as it might get invalidated"""
	def __init__(self, data_ptr):
		self.data_ptr = data_ptr

	def read_bytes(self, in_ptr: Ptr, in_len: Size) -> bytes:
		"""Read exactly `in_len` bytes from the memory at `in_ptr`."""
		return bytes(self.data_ptr[in_ptr : in_ptr + in_len])

	def write_bytes(self, out_ptr: Ptr, out_len: Size, data: bytes) -> int:
		"""Write up to `out_len` bytes from `data` to `out_ptr`."""
		count = min(out_len, len(data))
		for i in range(count):
			self.data_ptr[out_ptr + i] = data[i]
		
		return count
	
	def read_i32(self, in_ptr: Ptr) -> int:
		return struct.unpack("<i", self.read_bytes(in_ptr, 4))[0]

	def write_i32(self, out_ptr: Ptr, value: int):
		# pack as little endian signed integer
		# see https://docs.python.org/3/library/struct.html#byte-order-size-and-alignment
		self.write_bytes(out_ptr, 4, struct.pack("<i", value))

class WasiApp:
	@dataclass
	class _AppCore:
		instance: Instance
		setup_fn: Callable[[], None]
		teardown_fn: Callable[[], None]
		perform_fn: Callable[[], None]
		get_metrics_fn: Callable[[], int]
		clear_metrics_fn: Callable[[], None]
		get_devleoper_dump_fn: Callable[[], int]
	
	@dataclass
	class _PerformState:
		profile_url: str
		provider_url: str
		map_url: str
		usecase: str
		input: Any
		parameters: Mapping[str, str]
		security: SecurityValuesMap
		result: Optional[Any] = None
		error: Optional[PerformError] = None
		exception: Union[None, UnexpectedError, ValidationError] = None

	def __init__(
		self,
		filesystem: PythonFilesystem,
		network: PythonNetwork,
		persistence: PythonPersistence
	):
		self._engine = Engine()
		self._linker = Linker(self._engine)
		self._store = Store(self._engine)

		# linked modules and state
		self._linker.define_wasi()
		sf_host_link(self._linker, weakref.ref(self))

		wasi = WasiConfig()
		wasi.inherit_stdout()
		wasi.inherit_stderr()
		wasi.env = [*os.environ.items(), ('ONESDK_DEFAULT_USERAGENT', WasiApp.user_agent())]
		
		self._store.set_wasi(wasi)
		self._streams: HandleMap[BinaryIO] = HandleMap()
		self._requests: HandleMap[DeferredHttpResponse] = HandleMap()

		# dependencies
		self._filesystem = filesystem
		self._network = network
		self._persistence = persistence

		# loaded when core is loaded
		self._module: Optional[Module] = None
		self._core: Optional[WasiApp._AppCore] = None
		self._perform_state: Optional[WasiApp._PerformState] = None

	def _memory_from_core(self, core: "WasiApp._AppCore") -> WasiMemory:
		memory = cast(Memory, core.instance.exports(self._store)["memory"])
		ptr = memory.data_ptr(self._store)

		return WasiMemory(ptr)

	@property
	def memory(self) -> WasiMemory:
		if self._core is None:
			raise UninitializedError()

		return self._memory_from_core(self._core)

	def handle_message(self, message: Any) -> Any:
		if self._perform_state is None:
			raise UnexpectedError("UnexpectedError", "Unexpected perform state")
		
		if message["kind"] == "perform-input":
			return {
				"kind": "ok",
				"profile_url": self._perform_state.profile_url,
				"provider_url": self._perform_state.provider_url,
				"map_url": self._perform_state.map_url,
				"usecase": self._perform_state.usecase,
				"map_input": self._perform_state.input,
				"map_parameters": self._perform_state.parameters,
				"map_security": self._perform_state.security,
			}
		elif message["kind"] == "perform-output-result":
			self._perform_state.result = message["result"]
			return { "kind": "ok" }
		elif message["kind"] == "perform-output-error":
			self._perform_state.error = PerformError(message["error"])
			return { "kind": "ok" }
		elif message["kind"] == "perform-output-exception":
			if message["exception"]["error_code"] == "InputValidationError":
				self._perform_state.exception = ValidationError(message["exception"]["message"])
			else:
				self._perform_state.exception = UnexpectedError(message["exception"]["error_code"], message["exception"]["message"])
			return { "kind": "ok" }
		elif message["kind"] == "file-open":
			try:
				file_handle = self._filesystem.open(
					message["path"],
					create_new = message["create_new"],
					create = message["create"],
					truncate = message["truncate"],
					append = message["append"],
					write = message["write"],
					read = message["read"]
				)
			except WasiError as e:
				return { "kind": "err", "errno": e.errno }
			
			handle = self._streams.insert(
				# anonymous class/object with read, write and close methods
				cast(
					BinaryIO,
					SimpleNamespace(
						read = functools.partial(self._filesystem.read, file_handle),
						write = functools.partial(self._filesystem.write, file_handle),
						close = functools.partial(self._filesystem.close, file_handle)
					)
				)
			)
			return { "kind": "ok", "stream": handle }
		elif message["kind"] == "http-call":
			try:
				request = self._network.fetch(
					message["url"],
					message["method"],
					message["headers"],
					None if message["body"] is None else bytes(message["body"])
				)
			except HostError as err:
				return { "kind": "err", "error_code": err.code, "message": err.message }

			handle = self._requests.insert(request)
			return { "kind": "ok", "handle": handle }
		elif message["kind"] == "http-call-head":
			request = self._requests.remove(message["handle"])
			if request is None:
				return { "kind": "err", "error_code": ErrorCode.NetworkError, "message": "Invalid http call handle" }

			try:
				response = request.resolve()
			except HostError as err:
				return { "kind": "err", "error_code": err.code, "message": err.message }

			return {
				"kind": "ok",
				"status": response.status(),
				"headers": response.headers(),
				"body_stream": self._streams.insert(response.body())
			}
		else:
			return { "kind": "err", "error": f"Unknown message {message['kind']}" }
	
	def stream_read(self, handle: int, count: int) -> bytes:
		stream = self._streams.get(handle)
		if stream is None:
			raise WasiError(WasiErrno.EBADF)
		
		return stream.read(count)

	def stream_write(self, handle: int, data: bytes) -> int:
		stream = self._streams.get(handle)
		if stream is None:
			raise WasiError(WasiErrno.EBADF)
		
		return stream.write(data)
	
	def stream_close(self, handle: int):
		stream = self._streams.remove(handle)
		if stream is None:
			raise WasiError(WasiErrno.EBADF)
		
		stream.close()
	
	def load_core(self, wasm: bytes):
		self._module = Module(self._engine, wasm)

	def init(self):
		if self._module is None:
			raise UnexpectedError("CoreNotLoaded", "Call load_core first")

		if self._core is not None:
			return

		instance = self._linker.instantiate(self._store, self._module)
		exports = instance.exports(self._store)

		self._core = WasiApp._AppCore(
			instance,
			self._wrap_export(exports["oneclient_core_setup"]),
			self._wrap_export(exports["oneclient_core_teardown"]),
			self._wrap_export(exports["oneclient_core_perform"]),
			self._wrap_export(exports["oneclient_core_get_metrics"]),
			self._wrap_export(exports["oneclient_core_clear_metrics"]),
			self._wrap_export(exports["oneclient_core_get_developer_dump"])
		)

		self._core.setup_fn()
	
	def destroy(self):
		if self._core is not None:
			self.send_metrics()
			self._core.teardown_fn()
			self._core = None
	
	def perform(
		self,
		profile_url: str,
		provider_url: str,
		map_url: str,
		usecase: str,
		input: Any,
		parameters: Mapping[str, str],
		security: SecurityValuesMap
	) -> Any:
		if self._core is None:
			raise UninitializedError()

		self._perform_state = WasiApp._PerformState(
			profile_url = profile_url,
			provider_url = provider_url,
			map_url = map_url,
			usecase = usecase,
			input = input,
			parameters = parameters,
			security = security
		)

		self._core.perform_fn()
		
		state = self._perform_state
		self._perform_state = None

		if state.exception is not None:
			raise state.exception

		if state.error is not None:
			raise state.error

		return state.result

	def _wrap_export(self, fn: Any) -> Any:
		def wrapper(*args):
			try:
				return fn(self._store, *args)
			except Exception as e:
				err_name = "UnexpectedError"
				if isinstance(e, wasmtime.Trap):
					err_name = "WebAssemblyRuntimeError"

				if self._core is not None:
					core = self._core
					self._core = None

					try:
						self._create_developer_dump(core)
						self._send_metrics_on_panic(core)
					except Exception as dump_e:
						raise UnexpectedError(err_name, f"Error during dumping") from dump_e
				
				raise UnexpectedError(err_name, "Error while executing WebAssembly") from e
		
		return wrapper
	
	def _get_tracing_events_by_arena(self, core: "WasiApp._AppCore", arena_ptr: Ptr) -> List[str]:
		memory = self._memory_from_core(core)
		buffer1_ptr = memory.read_i32(arena_ptr)
		buffer1_size = memory.read_i32(arena_ptr + 4)
		buffer2_ptr = memory.read_i32(arena_ptr + 8)
		buffer2_size = memory.read_i32(arena_ptr + 12)

		buffer = memory.read_bytes(buffer1_ptr, buffer1_size) + memory.read_bytes(buffer2_ptr, buffer2_size)
		events = []

		start = 0
		while True:
			null_index = buffer.find(0, start)
			if null_index == -1:
				break

			events.append(buffer[start : null_index].decode("utf-8"))
			start = null_index + 1
		
		return events

	def send_metrics(self):
		if self._core is None:
			return
		
		arena_pointer = self._core.get_metrics_fn()
		events = self._get_tracing_events_by_arena(self._core, arena_pointer)
		self._core.clear_metrics_fn()
		
		if len(events) > 0:
			self._persistence.persist_metrics(events)

	def _send_metrics_on_panic(self, core: "WasiApp._AppCore"):
		arena_pointer = core.get_metrics_fn()
		events = self._get_tracing_events_by_arena(core, arena_pointer)
		
		if len(events) > 0:
			self._persistence.persist_metrics(events)

	def _create_developer_dump(self, core: "WasiApp._AppCore"):
		arena_pointer = core.get_devleoper_dump_fn()
		events = self._get_tracing_events_by_arena(core, arena_pointer)
		
		if len(events) > 0:
			self._persistence.persist_developer_dump(events)

	@staticmethod
	def user_agent():
		sys_platform = platform.system()
		sys_arch = platform.architecture()[0]
		python_version = sys.version.split()[0]

		return f"one-sdk-python/{metadata.version('one-sdk')} ({sys_platform} {sys_arch}) python/{python_version}"



from typing import TYPE_CHECKING, Tuple
from weakref import ReferenceType
if TYPE_CHECKING:
	from one_sdk.app import WasiApp

import sys
import json
import functools

from wasmtime import Linker, FuncType, ValType

from one_sdk.handle_map import HandleMap
from one_sdk.error import WasiError, WasiErrno

# TODO: TypeAlias - needs 3.10
Ptr = int
Size = int
AbiResult = int

def _strace_inner(fn, name, *args):
	result = fn(*args)
	print(f"host: [strace] {name}{args} = {result}", file = sys.stderr)
	return result
def strace(fn, name):
	"""Use on a function to wrap with a debug print when called."""
	return functools.partial(_strace_inner, fn, name)

def _join_abi_result(lower: int, upper: int) -> AbiResult:
	lower = lower & 0x7FFFFFFF
	upper = (upper & 0x1) << 31

	return lower | upper
def _split_abi_result(value: AbiResult) -> Tuple[int, int]:
	lower = value & 0x7FFFFFFF
	upper = (value >> 31) & 0x1

	return (lower, upper)
def _abi_ok(value: int) -> AbiResult:
	return _join_abi_result(value, 0)
def _abi_err(value: int) -> AbiResult:
	return _join_abi_result(int(value), 1)

# We need to use a weak reference because otherwise we cause a reference cycle with wasmtime runtime
# and leak the whole instance
# TODO: from 3.9 we can do ReferenceType["WasiApp"]
def _get_app(app_ref: ReferenceType) -> "WasiApp":
	app = app_ref()
	if app is None:
		raise RuntimeError("WasiApp already destroyed")
	
	return app

def link(linker: Linker, app_ref: ReferenceType):
	message_store: HandleMap[bytes] = HandleMap()

	def __export_message_exchange(msg_ptr: Ptr, msg_len: Size, out_ptr: Ptr, out_len: Size, ret_handle: Ptr) -> Size:
		app = _get_app(app_ref)

		memory = app.memory
		message = json.loads(
			memory.read_bytes(msg_ptr, msg_len).decode("utf-8")
		)
		response_bytes = json.dumps(app.handle_message(message)).encode("utf-8")

		message_handle = 0
		if len(response_bytes) > out_len:
			message_handle = message_store.insert(response_bytes)
		else:
			memory.write_bytes(out_ptr, out_len, response_bytes)
		
		memory.write_i32(ret_handle, message_handle)
		return _abi_ok(len(response_bytes))

	def __export_message_exchange_retrieve(handle: int, out_ptr: Ptr, out_len: Size) -> AbiResult:
		app = _get_app(app_ref)
		
		response_bytes = message_store.remove(handle)
		if response_bytes is None:
			return _abi_err(WasiErrno.EBADF)
		if out_len < len(response_bytes):
			return _abi_err(WasiErrno.ERANGE)
		
		count = app.memory.write_bytes(out_ptr, out_len, response_bytes)
		return _abi_ok(count)

	def __export_stream_read(handle: int, out_ptr: Ptr, out_len: Size) -> AbiResult:
		app = _get_app(app_ref)
		
		try:
			read_count = app.memory.write_bytes(out_ptr, out_len, app.stream_read(handle, out_len))
		except WasiError as e:
			return _abi_err(e.errno)
		
		return _abi_ok(read_count)
	
	def __export_stream_write(handle: int, in_ptr: Ptr, in_len: Size) -> AbiResult:
		app = _get_app(app_ref)
		
		try:
			write_count = app.stream_write(handle, app.memory.read_bytes(in_ptr, in_len))
		except WasiError as e:
			return _abi_err(e.errno)
		
		return _abi_ok(write_count)

	def __export_stream_close(handle: int) -> AbiResult:
		app = _get_app(app_ref)
		
		try:
			app.stream_close(handle)
		except WasiError as e:
			return _abi_err(e.errno)
		
		return _abi_ok(WasiErrno.SUCCESS)
	
	linker.define_func(
		"sf_host_unstable", "message_exchange",
		FuncType([ValType.i32(), ValType.i32(), ValType.i32(), ValType.i32(), ValType.i32()], [ValType.i32()]),
		__export_message_exchange
	)
	linker.define_func(
		"sf_host_unstable", "message_exchange_retrieve",
		FuncType([ValType.i32(), ValType.i32(), ValType.i32()], [ValType.i32()]),
		__export_message_exchange_retrieve
	)
	linker.define_func(
		"sf_host_unstable", "stream_read",
		FuncType([ValType.i32(), ValType.i32(), ValType.i32()], [ValType.i32()]),
		__export_stream_read
	)
	linker.define_func(
		"sf_host_unstable", "stream_write",
		FuncType([ValType.i32(), ValType.i32(), ValType.i32()], [ValType.i32()]),
		__export_stream_write
	)
	linker.define_func(
		"sf_host_unstable", "stream_close",
		FuncType([ValType.i32()], [ValType.i32()]),
		__export_stream_close
	)



from typing import Generic, MutableMapping, Optional, TypeVar

T = TypeVar("T")
class HandleMap(Generic[T]):
	def __init__(self):
		self._index: int = 1
		self.data: MutableMapping[int, T] = {}
	
	def insert(self, value: T) -> int:
		handle = self._index
		self._index += 1
		self.data[handle] = value

		return handle
	
	def get(self, handle: int) -> Optional[T]:
		return self.data.get(handle, None)
	
	def remove(self, handle: int) -> Optional[T]:
		return self.data.pop(handle, None)



from typing import Any

from enum import IntEnum, Enum

class BaseError(Exception):
	def __init__(self, name: str, message: str):
		self.name = name
		self.message = message
	
	def __str__(self) -> str:
		return f"{self.name}: {self.message}"

class PerformError(BaseError):
	def __init__(self, error_result: Any):
		super().__init__("PerformError", str(error_result))
		self.error_result = error_result

class ValidationError(BaseError):
	def __init__(self, message: str):
		super().__init__("ValidationError", message)

class UnexpectedError(BaseError):
	def __init__(self, name: str, message: str):
		super().__init__(name, message)

class UninitializedError(BaseError):
	def __init__(self):
		super().__init__("Uninitialized", "OneClient isn't initialized.")

class WasiErrno(IntEnum):
	SUCCESS = 0
	E2BIG = 1
	EACCES = 2
	EADDRINUSE = 3
	EADDRNOTAVAIL = 4
	EAFNOSUPPORT = 5
	EAGAIN = 6
	EALREADY = 7
	EBADF = 8
	EBADMSG = 9
	EBUSY = 10
	ECANCELED = 11
	ECHILD = 12
	ECONNABORTED = 13
	ECONNREFUSED = 14
	ECONNRESET = 15
	EDEADLK = 16
	EDESTADDRREQ = 17
	EDOM = 18
	EDQUOT = 19
	EEXIST = 20
	EFAULT = 21
	EFBIG = 22
	EHOSTUNREACH = 23
	EIDRM = 24
	EILSEQ = 25
	EINPROGRESS = 26
	EINTR = 27
	EINVAL = 28
	EIO = 29
	EISCONN = 30
	EISDIR = 31
	ELOOP = 32
	EMFILE = 33
	EMLINK = 34
	EMSGSIZE = 35
	EMULTIHOP = 36
	ENAMETOOLONG = 37
	ENETDOWN = 38
	ENETRESET = 39
	ENETUNREACH = 40
	ENFILE = 41
	ENOBUFS = 42
	ENODEV = 43
	ENOENT = 44
	ENOEXEC = 45
	ENOLCK = 46
	ENOLINK = 47
	ENOMEM = 48
	ENOMSG = 49
	ENOPROTOOPT = 50
	ENOSPC = 51
	ENOSYS = 52
	ENOTCONN = 53
	ENOTDIR = 54
	ENOTEMPTY = 55
	ENOTRECOVERABLE = 56
	ENOTSOCK = 57
	ENOTSUP = 58
	ENOTTY = 59
	ENXIO = 60
	EOVERFLOW = 61
	EOWNERDEAD = 62
	EPERM = 63
	EPIPE = 64
	EPROTO = 65
	EPROTONOSUPPORT = 66
	EPROTOTYPE = 67
	ERANGE = 68
	EROFS = 69
	ESPIPE = 70
	ESRCH = 71
	ESTALE = 72
	ETIMEDOUT = 73
	ETXTBSY = 74
	EXDEV = 75
	ENOTCAPABLE = 76

class WasiError(Exception):
	def __init__(self, errno: WasiErrno):
		self.message = f"WASI error: {errno.name}"
		self.errno = errno

# TODO: StrEnum base class - needs 3.11
class ErrorCode(str, Enum):
	NetworkError = "network:error"
	NetworkConnectionRefused = "network:ECONNREFUSED"
	NetworkHostNotFound = "network:ENOTFOUND"
	NetworkInvalidUrl = "network:invalid_url"

class HostError(Exception):
	def __init__(self, code: ErrorCode, message: str):
		self.code = code
		self.message = message



from typing import Any, Mapping, Optional, Union

import os
import os.path

from one_sdk.app import WasiApp, SecurityValuesMap
from one_sdk.error import UnexpectedError
from one_sdk.platform import PythonFilesystem, PythonNetwork, PythonPersistence

CORE_PATH = os.path.abspath(os.path.join(__file__, "../assets/core.wasm"))
if "CORE_PATH" in os.environ:
	CORE_PATH = os.environ["CORE_PATH"]

class InternalClient:
	def __init__(
		self,
		assets_path: str,
		token: Optional[str],
		superface_api_url: Union[None, str, bool]
	):
		self._assets_path = assets_path
		self._core_path = CORE_PATH
		self._ready = False
		self._file_system = PythonFilesystem()
		self._app = WasiApp(
			self._file_system,
			PythonNetwork(),
			PythonPersistence(token, superface_api_url, WasiApp.user_agent())
		)

	def resolve_profile_url(self, profile: str) -> str:
		resolved_profile = profile.replace('/', '.')
		path = os.path.abspath(os.path.join(self._assets_path, f"{resolved_profile}.profile.ts"))
		# migration from Comlink to TypeScript profiles
		path_comlink = os.path.abspath(os.path.join(self._assets_path, f"{resolved_profile}.profile"))
		if not self._file_system.exists(path) and self._file_system.exists(path_comlink):
			path = path_comlink

		return f"file://{path}"

	def resolve_map_url(self, profile: str, provider: str) -> str:
		resolved_profile = profile.replace('/', '.')
		path = os.path.abspath(os.path.join(self._assets_path, f"{resolved_profile}.{provider}.map.js"))

		return f"file://{path}"
	
	def resolve_provider_url(self, provider: str) -> str:
		path = os.path.abspath(os.path.join(self._assets_path, f"{provider}.provider.json"))

		return f"file://{path}"
	
	def init(self, core_path = None):
		if self._ready:
			return
		
		with open(self._core_path, "rb") as file:
			self._app.load_core(file.read())
		self._app.init()
		self._ready = True
	
	def destroy(self):
		if not self._ready:
			return
		
		self._app.destroy()
		self._ready = False
	
	def perform(
		self,
		profile: str,
		provider: str,
		usecase: str,
		input: Any,
		parameters: Mapping[str, str] = {},
		security: Optional[SecurityValuesMap] = None
	) -> Any:
		if security is None:
			security = dict()

		self.init()

		profile_url = self.resolve_profile_url(profile)
		provider_url = self.resolve_provider_url(provider)
		map_url = self.resolve_map_url(profile, provider)
		
		try:
			return self._app.perform(
				profile_url = profile_url,
				provider_url = provider_url,
				map_url = map_url,
				usecase = usecase,
				input = input,
				parameters = parameters,
				security = security
			)
		except UnexpectedError as e:
			if e.name == "WebAssemblyRuntimeError":
				self.destroy()
				self.init()
			raise
	
	def send_metrics(self):
		self._app.send_metrics()

class UseCase:
	def __init__(self, internal: InternalClient, profile: "Profile", name: str):
		self._internal = internal
		self._profile = profile
		self.name = name

	def perform(
		self,
		input: Any,
		provider: str,
		parameters: Mapping[str, str] = {},
		security: Optional[Mapping[str, Mapping[str, str]]] = None
	) -> Any:
		return self._internal.perform(
			profile = self._profile.name,
			provider = provider,
			usecase = self.name,
			input = input,
			parameters = parameters,
			security = security
		)

class Profile:
	def __init__(self, internal: InternalClient, name: str, url: str):
		self._internal = internal
		self.name = name
		self.url = url

	@classmethod
	def _load_local(cls, internal: InternalClient, name: str) -> "Profile": # TODO: Self return type - needs 3.11
		profile_url = internal.resolve_profile_url(name)
		return cls(internal, name, profile_url)

	
	def get_usecase(self, name: str) -> UseCase:
		return UseCase(self._internal, self, name)

class OneClient:
	def __init__(
		self,
		assets_path: str = "superface",
		token: Optional[str] = None,
		superface_api_url: Union[None, str, bool] = None
	):
		self._internal = InternalClient(
			assets_path = assets_path,
			token = token,
			superface_api_url = superface_api_url
		)
	
	def init(self):
		self._internal.init()
	
	def destroy(self):
		self._internal.destroy()
	
	def get_profile(self, name: str) -> Profile:
		return Profile._load_local(self._internal, name)
	
	def send_metrics_to_superface(self):
		self._internal.send_metrics()



import os
import unittest

from one_sdk import ValidationError
from one_sdk.app import WasiApp
from one_sdk.platform import PythonFilesystem, PythonNetwork, PythonPersistence

class TestApp(unittest.TestCase):
	@classmethod
	def setUpClass(cls):
		cls.app = WasiApp(
			filesystem = PythonFilesystem,
			network = PythonNetwork,
			persistence = PythonPersistence
		)

		with open(os.path.abspath(os.path.join(__file__, "../../src/one_sdk/assets/test-core.wasm")), "rb") as file:
			cls.app.load_core(file.read())
		cls.app.init()

	@classmethod
	def	tearDownClass(cls):
		cls.app.destroy()

	def test_invalid_user_input(self):
		with self.assertRaises(ValidationError):
			self.app.perform(
				profile_url = '',
				provider_url = '',
				map_url = '',
				usecase = 'CORE_PERFORM_INPUT_VALIDATION_ERROR',
				input = {},
				parameters = {},
				security = {}
			)

if __name__ == '__main__':
	unittest.main()
